//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "TexturedArea.hpp"
//------------------------------------------------------------------------------
#include <cmath>
#include <iostream>

#include "glUtils.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
TexturedArea::TexturedArea(Group *parent, const std::string &name) : Sprite(parent, name), m_areaSize(100, 100), m_textureSpeed(0, 0), m_textureOffset(0, 0)
{

}
//------------------------------------------------------------------------------
/** Get the current size of the textured area */
Vector2D TexturedArea::GetAreaSize() const
{
    return m_areaSize;
}
//------------------------------------------------------------------------------

/** Set the current size of the area */
void TexturedArea::SetAreaSize(const Vector2D &newSize)
{
    m_areaSize = newSize;
}
//------------------------------------------------------------------------------

/** Set the texture speed. This could be used to easily animate a textured aread. */
void TexturedArea::SetTextureSpeed(const Vector2D &speed)
{
    m_textureSpeed = speed;
}
//------------------------------------------------------------------------------

/** Get the texture speed. This could be used to easily animate a textured aread. */
Vector2D TexturedArea::GetTextureSpeed() const
{
    return m_textureSpeed;
}
//------------------------------------------------------------------------------

/** Set the offset of the texture. */
void TexturedArea::SetTextureOffset(const Vector2D &offset)
{
    m_textureOffset.x = floatingModulo(offset.x, GetSize().x);
    m_textureOffset.y = floatingModulo(offset.y, GetSize().y);

    if(m_textureOffset.x < 0.f)
    {
        m_textureOffset.x = GetSize().x - m_textureOffset.x;
    }

    if(m_textureOffset.y < 0.f)
    {
        m_textureOffset.y = GetSize().y - m_textureOffset.y;
    }
}
//------------------------------------------------------------------------------

/** Get the offset of the texture. */
Vector2D TexturedArea::GetTextureOffset() const
{
    return m_textureOffset;
}
//------------------------------------------------------------------------------

// Render
void TexturedArea::Render(sf::RenderTarget &App)
{

    if(IsVisible())
    {
        Vector2D pt2 = GetGlobalPosition() + m_areaSize;
        sf::FloatRect rect(GetGlobalPosition().x, GetGlobalPosition().y, pt2.x, pt2.y);

        GlCut tmpCut(rect, App);

        // Render the multuple images

        // Axis to draw the texture
        /*Vector2D axis1(GetSize().x, 0);
        Vector2D axis2(0, GetSize().y);

        // Rotation
        sf::Sprite::SetRotation(GetGlobalRotation() / (2 * M_PI) * 360.f);

        // Draw
        for(float x i = 0; i < m_areaSize.GetDotProduct(axis1); i++)
        {
            for(int j = 0; j < m_areaSize.GetDotProduct(axis2); j++)
            {
                sf::Sprite::SetPosition(m_textureOffset + axis1 * i + axis2 * j);
                App.Draw(*static_cast<const sf::Sprite*>(this));
            }
        }*/

        Vector2D pos = GetGlobalPosition() - GetSize() + m_textureOffset;

        // Rotation
        sf::Sprite::SetRotation(0);

        // Draw
        for(float x = 0; x < m_areaSize.x + GetSize().x; x += GetSize().x)
        {
            for(float y = 0; y < m_areaSize.y + GetSize().y; y += GetSize().y)
            {
                sf::Sprite::SetPosition(pos + Vector2D(x, y));
                App.Draw(*static_cast<const sf::Sprite*>(this));
            }
        }
    }
}
//------------------------------------------------------------------------------
void TexturedArea::ProceedMvt(float time)
{
    Sprite::ProceedMvt(time);


    // Move the texture
    SetTextureOffset(GetTextureOffset() + GetTextureSpeed() * time);
}
//------------------------------------------------------------------------------

} // namespace En
//------------------------------------------------------------------------------
