
------------------------------------------------------------
-- animBaseCreate
-- grp: The group used to protect the event
-- fn: The function to be called a each frame
-- duration: the duration of a loop / of the animation
-- isLoop: if the animation stop when the time is up or if it start again
-- isRestart: from 1 to 0 instead of 0 to 1
-- isRestart: do not try to continue an animation
-- userData: data to be send to the function fn

-- fn : (grp, ratio, evt, userData)
------------------------------------------------------------

function animBaseCreate(grp, fn, duration, isLoop, isReverse, isRestart, userData)
	-- Reprise
	local backRatio = nil
	if not isRestart then
		if grp:UserDataExists("_animBaseEvt") then
			local backEvt = grp:GetUserData("_animBaseEvt")
			if backEvt ~= nil then
				backRatio = backEvt:GetRatio()
				animBaseStop(backEvt)
			end
		end
	end
	
	-- Evt
	local evt = grp:GetParentScreen():RegisterChronometer(animEventPrivateOnChrono, duration, true)
	guiLib_addAutoDestroyEvent(grp, evt)
	grp:SetUserData("_animBaseEvt", evt)
	
	evt:SetUserData("_animBaseInfo", { ["isLoop"] = isLoop, ["isReverse"] = isReverse, ["isRestart"] = isRestart})
	evt:SetUserData("_animBaseUserData", userData)
	evt:SetUserData("_animBaseGroup", grp)
	evt:SetUserData("_animBaseFn", fn)
	
	-- Reprise
	if backRatio ~= nil then
		evt:SetRatio(backRatio)
	end
end

function animBaseStop(evt)
	local grp = evt:GetUserData("_animBaseGroup")

	guiLib_removeAutoDestroyEvent(grp, evt)
	evt:Destroy()
	evt = nil
	grp:SetUserData("_animBaseEvt", nil)
end

------------------------------------------------------------
-- Private
------------------------------------------------------------

function animEventPrivateOnChrono(ratio, evt)
	local grp = evt:GetUserData("_animBaseGroup")
	local userData = evt:GetUserData("_animBaseUserData")
	local info = evt:GetUserData("_animBaseInfo")
	local fn = evt:GetUserData("_animBaseFn")
	
	-- Destroy / loop
	if ratio == 1 then
		if info["isLoop"] then
			evt:Restart()
		else
			animBaseStop(evt)
		end
	end
	
	-- Reverse
	if info["isReverse"] then
		ratio = 1 - ratio
	end
	
	-- Evt
	fn(grp, ratio, evt, userData)
end

