
------------------------------------------------------------
-- Bombe
------------------------------------------------------------

function onBombeColide(phBombe, solOrPlayer, pos, penetrationVector)
	-- Check si sol ou joueur
	local bombe = phBombe:GetParent()
	
	if solOrPlayer:GetParent():UserDataExists("keyTbl") then
		local playerCatch = solOrPlayer:GetParent()
		local playerOrigin = bombe:GetUserData("origin")
		
		if playerCatch ~= playerOrigin then -- La bombe passe à travers le joueur qui l'a lancé
			-- Sound
			soundNs.PlaySnd(sndBombeCatch, gameScreen)
		
			-- Echange des rôles
			playerSetMode(playerCatch, "throw")
			
			animChoseOneImage(playerOrigin:FindByNameSprite("spr"), 5)
			
			-- On laisse les particules disparaitre
			bombeMigrateParticle(bombe)
			
			-- On la détruit
			bombe:Destroy()
			
			-- Zoom
			adaptZoom({playerCatch, playerOrigin})
		end
	else
		local sol = solOrPlayer
		
		if UserDataExistsAndIsEqual(sol, "_isRebound", true) then
			-- On fait sortir la bombe du mur
			bombe:Move(penetrationVector)
			
			-- On inverse la vitesse selon la normal au choc
			local penetrationDirection = penetrationVector:GetDirection()
			bombe:SetSpeed(bombe:GetSpeed() - penetrationDirection * bombe:GetSpeed():GetDotProduct(penetrationDirection) * 2)
			
			-- On multiplie la vitesse par rebound speed
			bombe:SetSpeed(bombe:GetSpeed() * sol:GetUserData("_reboundForce"))
			
			
			-- Snd
			soundNs.PlayGlobalSnd(sndBombeRebound)
		
			--bombe:Move(bombe:GetSpeed() * GetFrameTime() * -1)
			--bombe:SetSpeed(Vector2D(bombe:GetSpeed().x * -1, bombe:GetSpeed().y))
		elseif UserDataExistsAndIsEqual(sol, "_isBombePass", true) then -- Pas de collision avec la bombe
			-- Rien
		else					
			-- Clear trajectoire
			clearBombeTrajectoire()
			
			-- On reprend un checkpoint, si ce n'est pas déjà fait avec cette collision	
			if isLevelChallenge() then
				-- malus
				timeAddMalus(timeMalusBombeExplode)
				
				-- Le joueur reprend la bombe
				playerSetMode(bombe:GetUserData("origin"), "throw")
				
				-- Explosion
				CreateExplosion(bombe:GetGlobalPosition(), math.pi / 2, bombe:GetSpeed():GetDirection(), true)
				-- Son
				soundNs.PlayGlobalSnd(sndBombeBurst)
			else
				onPlayerKilled(bombe:GetUserData("origin"), "bombeWall", bombe)
			end
			
			-- On laisse les particules disparaitre
			bombeMigrateParticle(bombe)
			
			-- On la détruit
			bombe:Destroy()
		end
	end
end

function createMovingBombe(parentGroup, direction, speed, player)
	local bombeGrp = parentGroup:CreateGroup("bombeGrp")
	bombeGrp:SetPosition(player:GetPosition() + viseurCenterPos)
	bombeGrp:SetUserData("isOnAir", true)
	startGravity(bombeGrp, true)
	
	-- Sound
	soundNs.PlaySnd(sndBombeLaunch, gameScreen)
	
	-- Origine
	bombeGrp:SetUserData("origin", player)
	
	-- Physic
	local ph = bombeGrp:CreatePhCircle(phGroupBombe, "phBombe", Vector2D(0, 0), bombeRadius)
	ph:SetEventFunction(onBombeColide)
	
	-- Design
	--local tmp = bombeGrp:CreateShapeCircle("design", Vector2D(0, 0), bombeRadius, Color(150, 0, 0), 1, Color(0, 0, 0))
	local tmp = bombeGrp:CreateSprite("design", bombeTex, bombeTex:GetSize() / -2)
	
	-- Vitesse et direction
	bombeGrp:SetPosition(player:GetPosition() + viseurCenterPos)
	bombeGrp:SetSpeed(direction * speed)
	
	-- Rotation
	bombeGrp:SetAngularVelocity((math.random() - 0.5) * 40) -- -20 < x < 20
	
	-- Particle
	local model = ParticleModel(0.3, 0.8)
	model:AddMutableParam("alpha", 0.7, 0)
	model:AddMutableRandomParam("blue", 0.3, 0.5, 0, 0.1)
	model:AddMutableRandomParam("red", 0.8, 1, 0.7, 1)
	model:AddMutableRandomParam("green", 0.8, 1, 0, 0.2)
	model:AddMutableRandomParam("size", 0.7, 1, 1.5, 2)

	local emitter = ParticleEmitter()
	emitter:MakeDisk(Vector2D(0, 0), 4)
	emitter:SetAngles(3.14 / 4, 0)
	emitter:SetDirection(Vector2D(0, -1))

	-- Particle
	local particleObj = bombeGrp:CreateParticleSource("particle", model, 24)
	particleObj:MakeRenderImages(texParticlePoint, Vector2D(40, 40), true)
	particleObj:SetEmitter(emitter)
	particleObj:SetFlow(30)
	particleObj:SetForce(30, 100)
	particleObj:SetFriction(3)
	particleObj:SetGravity(Vector2D(0, -15))
	particleObj.position = Vector2D(0, 0)
	particleObj.zPos = -50

	-- return
	return bombeGrp
end

function bombeMigrateParticle(bombeGrp)
	local particleObj = bombeGrp:FindByName("particle")
	
	local backPos = particleObj:GetGlobalPosition()
	
	particleObj:ChangeParent(levelGrp)
	particleObj:SetGlobalPosition(backPos)
	particleObj:SetFlow(0)
	particleObj:SetAutoDestroy(true)
end
