------------------------------------------------------------
-- Namespace
------------------------------------------------------------

lvlProgress = lvlProgress or {}
lvlProgress.priv = lvlProgress.priv or {}


------------------------------------------------------------
-- Fns
------------------------------------------------------------
function lvlProgress.init()
	local tmpNode = GetConfigManager():GetRootConfigNode():AccessNode("default"):AccessNode("levelProgress")
	
	lvlProgress.profilNodeNormal = tmpNode:AccessNode("campagne")
	lvlProgress.profilNodeChallenge = tmpNode:AccessNode("challenge")
end

-- isLevelDone
function lvlProgress.isLevelDone(lvlName, isChallenge)
	local node = lvlProgress.priv.getLevelNode(lvlName, isChallenge)
	
	if node:ChildExists("done") then
		return (node:GetValue("done") == true)
	else
		return false
	end
end

-- getLevelStats
function lvlProgress.getLevelStats(lvlName, isChallenge)
	local node = lvlProgress.priv.getLevelNode(lvlName, isChallenge):AccessNode("stats")
	
	return node:GetLuaTable()
end


-- Add level stats
function lvlProgress.addLevelStats(lvlName, lvlStats, isChallenge)
	local node = lvlProgress.priv.getLevelNode(lvlName, isChallenge):AccessNode("stats")
	
	-- Stats
	originTbl = node:GetLuaTable()
	
	if (originTbl == nil) or (next(originTbl) == nil) then
		originTbl = lvlStats
	else
		originTbl.minTime = math.min(lvlStats.minTime, originTbl.minTime)
		if not isChallenge then
			originTbl.minNbLaunch = math.min(lvlStats.minNbLaunch, originTbl.minNbLaunch)
			originTbl.minDeath = math.min(lvlStats.minDeath, originTbl.minDeath)
		end
	end
	
	node:Clear()
	node:SetLuaTable(originTbl)
	
	-- Done
	lvlProgress.priv.getLevelNode(lvlName, isChallenge):SetValue("done", true)
end

-- Format level stats
function lvlProgress.getStatsText(lvlStats, isInMap, isChallenge, baseChallengeTime)
	local str = ""

	if next(lvlStats) == nil then -- Table empty
		str = _("Not done yet")
	else
		if isInMap then
			str = str .. "Meilleur score :\n"
		end
		str = str .. "Temps : " .. lvlProgress.priv.timeFormat(lvlStats.minTime)
		if not isChallenge then
			str = str .. "\nLancers : " .. tostring(lvlStats.minNbLaunch)
			str = str .. "\nMorts : " .. tostring(lvlStats.minDeath)
		else
			if lvlStats.minTime <= baseChallengeTime * goldRankRatio then
				str = str .. "\n" .. _("You have the gold medal !")
			elseif lvlStats.minTime <= baseChallengeTime * silverRankRatio then
				str = str .. "\n" .. _("You have the silver medal !")
			elseif lvlStats.minTime <= baseChallengeTime * bronzeRankRatio then
				str = str .. "\n" .. _("You have the bronze medal !")
			else
				str = str .. "\n" .. _("Sorry, you haven't been fast enough !")
			end
		end
	end

	return str
end

------------------------------------------------------------
-- Private
------------------------------------------------------------

-- Get the level node
function lvlProgress.priv.getLevelNode(lvlName, isChallenge)
	if isChallenge == nil then
		printErrorConsole("isChallenge = nil")
		return
	end

	if isChallenge then
		return lvlProgress.profilNodeChallenge:AccessNode(lvlName)
	else
		return lvlProgress.profilNodeNormal:AccessNode(lvlName)
	end
end

-- Default if nil
function lvlProgress.priv.defaultIfNil(value, default)
	if value == nil then
		return default
	else
		return value
	end
end

-- Time format
function lvlProgress.priv.timeFormat(seconds)
	return lvlProgress.priv.formatTimeNumber(seconds / 60) .. "'" .. lvlProgress.priv.formatTimeNumber(seconds % 60) .. "\"" .. lvlProgress.priv.formatTimeNumber((seconds * 100) % 100)
end

function lvlProgress.priv.formatTimeNumber(value)
	local str = tostring(math.floor(value))
	
	if string.len(str) == 1 then -- On met 2 chiffres
		str = "0" .. str
	end
	
	return str
end

------------------------------------------------------------
-- Init
------------------------------------------------------------
lvlProgress.init()
