
------------------------------------------------------------
-- Change the screen, change the music events and add a transition
------------------------------------------------------------

function myChangeScreen(newScreen, isTransition, transitionId, transitionScreen, userData)
	
	-- Migrate music playlist event
	musicNs.changeScreen(newScreen)
	
	-- Transition
	if isTransition == true then
		-- Default
		if transitionScreen == nil then
			transitionScreen = newScreen
		end
		
		if transitionId == nil then
			transitionId = "slide1"
		end
		
		-- Transition Id
		changeScreenNs.transitionId = transitionId
	
		if (changeScreenNs.transitionId == "slide1") or (changeScreenNs.transitionId == "slide2") then
			changeScreenNs.createScreenTransition1(GetCurrentScreen(), transitionScreen, newScreen, userData)
		elseif changeScreenNs.transitionId == "fade" then -- Fade
			changeScreenNs.createScreenTransitionFade(GetCurrentScreen(), transitionScreen, newScreen, userData)
		else
			printErrorConsole("myChangeScreen: Unknown ransitionId '" .. tostring(transitionId) .. "'.")
		end
	else	
		-- Change the screen
		ChangeScreen(newScreen)
	end
end


------------------------------------------------------------
-- Private
------------------------------------------------------------

changeScreenNs = changeScreenNs or {}

changeScreenNs.transitionId = nil


------------------------------------------------------------
-- Slide
------------------------------------------------------------
function changeScreenNs.createScreenTransition1(backScreen, transitionScreen, newScreen)
	-- Snd
	soundNs.PlayGlobalSnd(sndMenuSlide)

	-- Set screens
	changeScreenNs.backScreenRenderer:SetTargetScreen(backScreen)
	changeScreenNs.backScreenRenderer:SetScreenConfig(backScreen:GetScreenConfig())
	changeScreenNs.backScreenRenderer:SetZpos(-50)
	
	changeScreenNs.newScreenRenderer:SetTargetScreen(transitionScreen)
	changeScreenNs.newScreenRenderer:SetScreenConfig(transitionScreen:GetScreenConfig())
	changeScreenNs.newScreenRenderer:SetZpos(50)
	
	-- Transition
	animBaseCreate(changeScreenNs.transitionScreen, changeScreenNs.onScreenTransition1Ratio, slideTransitionTime, false, false, true, newScreen)
	changeScreenNs.onScreenTransition1Ratio(nil, 0, nil, newScreen)
	
	-- Change to animation screen
	myChangeScreen(changeScreenNs.transitionScreen, false)
end

function changeScreenNs.onScreenTransition1Ratio(grp, ratio, evt, userDataNewScreen)
	-- End
	if ratio == 1 then
		myChangeScreen(userDataNewScreen, false)
	else -- Anim
		ratio = ConvertRatioCosAccelerationDeceleration(ratio)
	
		-- ratio
		if changeScreenNs.transitionId == "slide2" then -- Other way
			-- Back
			local topLeft = Vector2D(-1 * screenSize.x * ratio, 0)
			
			local cfg = ScreenConfig(screenSize)
			cfg:SetCameraTopLeft(topLeft)
			cfg:SetCameraBottomRight(topLeft + screenSize)
			changeScreenNs.backScreenRenderer:SetScreenConfig(cfg)
			
			changeScreenNs.transitionScreen:FindByName("sreenBackGrp"):SetCutRect(topLeft * -1, screenSize)
			
			-- New
			topLeft = Vector2D((1 - ratio) * screenSize.x, 0)
			
			cfg = ScreenConfig(screenSize)
			cfg:SetCameraTopLeft(topLeft)
			cfg:SetCameraBottomRight(topLeft + screenSize)
			changeScreenNs.newScreenRenderer:SetScreenConfig(cfg)
			
			changeScreenNs.transitionScreen:FindByName("sreenNewGrp"):SetCutRect(Vector2D(0, 0), screenSize - topLeft)
		else
			-- Back
			local topLeft = Vector2D(screenSize.x * ratio, 0)
			
			local cfg = ScreenConfig(screenSize)
			cfg:SetCameraTopLeft(topLeft)
			cfg:SetCameraBottomRight(topLeft + screenSize)
			changeScreenNs.backScreenRenderer:SetScreenConfig(cfg)
			
			changeScreenNs.transitionScreen:FindByName("sreenBackGrp"):SetCutRect(Vector2D(0, 0), topLeft + screenSize)
			
			
			-- New
			topLeft = Vector2D( -1 * (1 - ratio) * screenSize.x, 0)
			
			cfg = ScreenConfig(screenSize)
			cfg:SetCameraTopLeft(topLeft)
			cfg:SetCameraBottomRight(topLeft + screenSize)
			changeScreenNs.newScreenRenderer:SetScreenConfig(cfg)
			
			changeScreenNs.transitionScreen:FindByName("sreenNewGrp"):SetCutRect(topLeft * -1, screenSize)
		end
	end
end


--[[
function changeScreenNs.createScreenTransition1(backScreen, newScreen)
	-- Set screens
	changeScreenNs.backScreenRenderer:SetTargetScreen(backScreen)
	changeScreenNs.backScreenRenderer:SetScreenConfig(backScreen:GetScreenConfig())
	changeScreenNs.backScreenRenderer:SetZpos(-50)
	
	changeScreenNs.newScreenRenderer:SetTargetScreen(newScreen)
	changeScreenNs.newScreenRenderer:SetScreenConfig(newScreen:GetScreenConfig())
	changeScreenNs.newScreenRenderer:SetZpos(50)
	
	-- Transition
	animBaseCreate(changeScreenNs.transitionScreen, changeScreenNs.onScreenTransition1Ratio, 0.5, false, false, true, nil)
	
	-- Change to animation screen
	myChangeScreen(changeScreenNs.transitionScreen, false)
end

function changeScreenNs.onScreenTransition1Ratio(grp, ratio, evt, userData)
	-- Back
	local topLeft = Vector2D(- screenSize.x * ratio, 0)
	
	local cfg = ScreenConfig(screenSize)
	cfg:SetCameraTopLeft(topLeft)
	cfg:SetCameraBottomRight(topLeft + screenSize)
	changeScreenNs.backScreenRenderer:SetScreenConfig(cfg)
	
	changeScreenNs.transitionScreen:FindByName("sreenBackGrp"):SetCutRect(topLeft, topLeft + screenSize)
	
	
	-- New
	topLeft = Vector2D((1 - ratio) * screenSize.x, 0)
	
	cfg = ScreenConfig(screenSize)
	cfg:SetCameraTopLeft(topLeft)
	cfg:SetCameraBottomRight(topLeft + screenSize)
	changeScreenNs.newScreenRenderer:SetScreenConfig(cfg)
	
	changeScreenNs.transitionScreen:FindByName("sreenNewGrp"):SetCutRect(topLeft, topLeft + screenSize)
	
	-- End
	if ratio == 1 then
		myChangeScreen(changeScreenNs.newScreenRenderer:GetTargetScreen(), false)
	end
end]]

------------------------------------------------------------
-- Fade
------------------------------------------------------------
function changeScreenNs.createScreenTransitionFade(backScreen, transitionScreen, newScreen, userData)
	userData = userData or {}
	userData.isHalfDone = false -- Pas arrivé à la moitier

	-- Snd
	soundNs.PlayGlobalSnd(sndFadeTransition)

	-- Set screens
	changeScreenNs.backScreenRenderer:SetTargetScreen(backScreen)
	changeScreenNs.backScreenRenderer:SetScreenConfig(backScreen:GetScreenConfig())
	local grp = changeScreenNs.transitionScreen:FindByName("sreenBackGrp")
	grp:SetCutRect(Vector2D(0, 0), screenSize)
	grp:Show(true)
	
	changeScreenNs.newScreenRenderer:SetTargetScreen(transitionScreen)
	changeScreenNs.newScreenRenderer:SetScreenConfig(transitionScreen:GetScreenConfig())
	local grp = changeScreenNs.transitionScreen:FindByName("sreenNewGrp")
	grp:SetCutRect(Vector2D(0, 0), screenSize)
	grp:Show(false)
	
	-- Fade
	local tmp = changeScreenNs.transitionScreen:CreateShapeRectangle("fadeRect", Vector2D(0, 0), screenSize, Color(0, 0, 0, 255), 0, Color(0, 0, 0))
	tmp:SetZpos(1000)
	
	-- Transition
	userData.newScreen = newScreen
	animBaseCreate(changeScreenNs.transitionScreen, changeScreenNs.onScreenTransitionFadeRatio, fadeTransitionTime, false, false, true, userData)
	changeScreenNs.onScreenTransitionFadeRatio(nil, 0, nil, newScreen)
	
	-- Change to animation screen
	myChangeScreen(changeScreenNs.transitionScreen, false)
end

function changeScreenNs.onScreenTransitionFadeRatio(grp, ratio, evt, userData)
	-- End
	if ratio == 1 then
		changeScreenNs.transitionScreen:FindByName("fadeRect"):Destroy()
		myChangeScreen(userData.newScreen, false)
		changeScreenNs.transitionScreen:FindByName("sreenBackGrp"):Show(true)
		changeScreenNs.transitionScreen:FindByName("sreenNewGrp"):Show(true)
	else -- Anim
		--ratio = ConvertRatioCosAccelerationDeceleration(ratio)
	
		-- ratio
		if ratio < 0.5 then
			ratio = ratio * 2
			changeScreenNs.transitionScreen:FindByName("fadeRect"):SetColor(Color(0, 0, 0, 255 * ratio))
		else
			if not userData.isHalfDone then
				changeScreenNs.transitionScreen:FindByName("sreenBackGrp"):Show(false)
				changeScreenNs.transitionScreen:FindByName("sreenNewGrp"):Show(true)
				ratio = 0.5 -- Completely black
				
				if userData.onHalfFn ~= nil then
					userData.onHalfFn()
				end
				
				userData.isHalfDone = true
			end
			
			ratio = 1 - ratio
			ratio = ratio * 2
			changeScreenNs.transitionScreen:FindByName("fadeRect"):SetColor(Color(0, 0, 0, 255 * ratio))
		end
	end
end

------------------------------------------------------------
-- Transition screen
------------------------------------------------------------

changeScreenNs.transitionScreen = CreateScreen("transitionScreen", ScreenConfig(screenSize))

-- Left
local tmp = changeScreenNs.transitionScreen:CreateGroup("sreenBackGrp")
tmp:SetCutRect(Vector2D(0, 0), screenSize)
tmp:EnableCut(true)
changeScreenNs.backScreenRenderer = tmp:CreateScreenRenderer("rendererBack")

-- Right
local tmp = changeScreenNs.transitionScreen:CreateGroup("sreenNewGrp")
tmp:SetCutRect(Vector2D(0, 0), screenSize)
tmp:EnableCut(true)
changeScreenNs.newScreenRenderer = tmp:CreateScreenRenderer("rendererNew")
